<?php

namespace Drupal\billwerk_subscriptions_subscriptionjs\Form;

use Drupal\billwerk_subscriptions_subscriptionjs\Form\BillwerkSubscriptionjsFormAbstract;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Session\AccountInterface;

/**
 * The premium downgrade form.
 */
class PlanVariantUpgradeForm extends BillwerkSubscriptionjsFormAbstract {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'billwerk_subscriptions_subscriptionjs_plan_variant_upgrade_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    // @todo
    $currentPlanVariantKey = 'zmp_live_plus_bronce_quarterly';
    $options = $this->subscriptionPlansManager->getPlanVariantsUpgradeSelectOptions($currentPlanVariantKey);
    if (!empty($options)) {
      $form['planvariant_current'] = [
        '#type' => 'item',
        '#title' => $this->t('Your current subscription plan:'),
        '#markup' => $this->subscriptionPlansManager->getBillwerkPlanVariantByMachineName($currentPlanVariantKey)->labelTranslated(),
      ];

      $form['planvariant_id'] = [
        '#type' => 'radios',
        '#title' => $this->t('Upgrade to'),
        '#options' => $options,
        '#description' => $this->t('Select the subscription plan for the upgrade'),
        '#required' => TRUE,
        '#ajax' => [
          'callback' => '::updatePlanDetailsCallback',
          'wrapper' => 'edit-plan-details',
          'progress' => [
            'type' => 'throbber',
            'message' => $this->t('Loading plan details ...'),
          ],
        ],
      ];

      $form['planvariant_details'] = [
        '#theme' => 'plan_variant_details',
        '#prefix' => '<div id="edit-plan-details">',
        '#suffix' => '</div>',
      ];
      if ($form_state->hasValue('planvariant_id')) {
        $planVariantKey = $form_state->getValue('planvariant_id');
        $form['planvariant_details']['#label'] = $this->subscriptionPlansManager->getBillwerkPlanVariantByMachineName($planVariantKey)->labelTranslated();
      }

      $form['actions']['#type'] = 'actions';
      $form['actions']['submit'] = [
        '#type' => 'submit',
        '#value' => $this->t('Buy now'),
        '#button_type' => 'secondary',
      ];
    }
    else {
      $form = [
        '#type' => 'item',
        '#markup' => $this->t('No upgrade possible for your subscription. Should you have any questions, please contact us.'),
      ];
    }

    return $form;
  }

  /**
   * AJAX Callback for the plan variants details.
   */
  public function updatePlanDetailsCallback(array &$form, FormStateInterface $form_state) {
    return $form['planvariant_details'];
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // @todo Execute the required JS action or redirect.
  }

  /**
	 * Checks access for a specific request.
	 *
	 * @param \Drupal\Core\Session\AccountInterface $account
	 *   Run access checks for this account.
	 *
	 * @return \Drupal\Core\Access\AccessResultInterface
	 *   The access result.
	 */
	public static function access(AccountInterface $account) {
    // @todo: Implement!
		return AccessResult::allowed();
	}

}
