<?php

namespace Drupal\billwerk_subscriptions_subscriptionjs\Form;

use Drupal\billwerk_subscriptions\Subscriber;
use Drupal\billwerk_subscriptions_subscriptionjs\Form\BillwerkSubscriptionjsFormAbstract;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Session\AccountInterface;

/**
 * The premium downgrade form.
 */
class SubscriptionForm extends BillwerkSubscriptionjsFormAbstract {

	/**
	 * {@inheritdoc}
	 */
	public function getFormId() {
		return 'billwerk_subscriptions_subscriptionjs_subscription_form';
	}

	/**
	 * {@inheritdoc}
	 */
	public function buildForm(array $form, FormStateInterface $form_state) {
		switch ($form_state->get('step')) {
			case 'review':
				$form = $this->buildReviewForm($form, $form_state);
				break;

			case 'buy':
				$form = $this->buildBuyForm($form, $form_state);
				break;

			default:
				$form = $this->buildSelectionForm($form, $form_state);
				break;
		}
		return $form;
	}

	protected function buildSelectionForm(array $form, FormStateInterface $form_state) {
		$subscriber = Subscriber::loadByDrupalUid($this->currentUser()->id());
		$subscriberCurrentPlanVariantId = $subscriber->getBillwerkContract()->getBillwerkContractSubscription()->getPlanVariantId();
		$subscriberCurrentPlanVariantEntity = $this->subscriptionPlansManager->getBillwerkPlanVariantByPlanVariantId($subscriberCurrentPlanVariantId, $this->environment);
		$subscriberCurrentComponentSubscriptions = $subscriber->getBillwerkContract()->getBillwerkContractSubscription()->getComponentSubscriptions();

		$form['planvariant_current'] = [
			'#type' => 'item',
			'#title' => $this->t('Your current subscription plan:'),
			'#markup' => $subscriberCurrentPlanVariantEntity->labelTranslated(),
		];

		$options = $this->subscriptionPlansManager->getPlanVariantsAllSelectOptions();
		if (!empty($options)) {
			// We're using the machine names as keys to not expose the Billwerk IDs:
			$form['planvariant'] = [
				'#type' => 'radios',
				'#title' => $this->t('Subscription'),
				'#options' => $options,
				'#default_value' => $subscriberCurrentPlanVariantEntity->getMachineName(),
				'#description' => $this->t('Select the subscription plan for the upgrade'),
				'#required' => TRUE,
				'#ajax' => [
					'callback' => '::updatePlanDetailsCallback',
					'wrapper' => 'edit-plan-details',
					'progress' => [
						'type' => 'throbber',
						'message' => $this->t('Loading plan details ...'),
					],
				],
			];

			$form['planvariant_details'] = [
				'#theme' => 'plan_variant_details',
				'#prefix' => '<div id="edit-plan-details">',
				'#suffix' => '</div>',
			];
			if ($form_state->hasValue('planvariant')) {
				$planVariantKey = $form_state->getValue('planvariant');
				$form['planvariant_details']['#label'] = $this->subscriptionPlansManager->getBillwerkPlanVariantByMachineName($planVariantKey)->labelTranslated();
			}

			// Components:
			$currentAddons = [];
			if (!empty($subscriberCurrentComponentSubscriptions)) {
				foreach ($subscriberCurrentComponentSubscriptions as $subscriberCurrentComponentSubscription) {
					$subscriberCurrentPlanVariantEntity = $this->subscriptionPlansManager->getBillwerkComponentByComponentId($subscriberCurrentComponentSubscription->getComponentId(), $this->environment);
					$currentAddons[] = $subscriberCurrentPlanVariantEntity->getMachineName();
				}
			}
			// We're using the machine names as keys to not expose the Billwerk IDs:
			$form['components'] = [
				'#type' => 'checkboxes',
				'#title' => $this->t('Addons'),
				'#options' => $this->subscriptionPlansManager->getComponentsAllSelectOptions(),
				'#default_value' => $currentAddons,
				'#multiple' => TRUE,
				'#description' => $this->t('Select the add-ons to add to your subscription.'),
				'#required' => TRUE,
			];

			$form['couponcode'] = [
				'#type' => 'textfield',
				'#title' => $this->t('Coupon code'),
				'#description' => $this->t('Do you have a coupon code?'),
				'#default_value' => '',
				'#required' => FALSE,
			];

			$form['actions']['#type'] = 'actions';
			$form['actions']['proceed'] = [
				'#type' => 'submit',
				'#value' => $this->t('Proceed'),
				'#submit' => ['::submitSelectionForm'],
				'#button_type' => 'primary',
			];
		}
		else {
			$form['message'] = [
				'#type' => 'item',
				'#markup' => $this->t('No change possible for your subscription. Please contact us.'),
			];
		}

		return $form;
	}

	protected function buildReviewForm(array $form, FormStateInterface $form_state): array {
    $subscriber = Subscriber::loadByDrupalUid($this->currentUser()->id());

		$form['actions']['#type'] = 'actions';
		$form['actions']['submit'] = [
			'#type' => 'submit',
      '#submit' => ['::submitReviewForm'],
			'#value' => $this->t('Proceed'),
			'#button_type' => 'primary',
		];

		$form['#attached']['library'][] = 'billwerk_subscriptions_subscriptionjs/subscriptionjs';
		$form['#attached']['library'][] = 'billwerk_subscriptions_subscriptionjs/subscriptionjs.init';
		$form['#attached']['drupalSettings']['billwerk']['selfserviceToken'] = $subscriber->billwerkGetSelfserviceToken();
		// @todo:
		$selectedPlanVariantMachineName = $form_state->getValue('planvariant');
		$newPlanVariantEntitiy = $this->subscriptionPlansManager->getBillwerkPlanVariantByMachineName($selectedPlanVariantMachineName);
		$selectedComponentsMachineNames = $form_state->getValue('components');
		if (!empty($selectedComponentsMachineNames)) {
			foreach ($selectedComponentsMachineNames as $selectedComponentsMachineName) {
				// $newComponentsEntities = $this->subscriptionPlansManager->getBillwerkComponentByComponentId($subscriberCurrentComponentSubscription->getComponentId(), $this->environment);
			}
		}
		$selectedCouponCode = $form_state->getValue('couponcode');

		// @todo: Finish:
		$billwerkOrderObj = $subscriber->billwerkChangeSubscription($newPlanVariantEntitiy->getBillwerkId($this->environment->getEnvironmentName()), [], [], $selectedCouponCode);
    dsm($billwerkOrderObj);
    $form['#attached']['drupalSettings']['billwerk']['orderId'] = $billwerkOrderObj->Id;

		return $form;
	}


  protected function buildBuyForm(array $form, FormStateInterface $form_state): array {
    $subscriber = Subscriber::loadByDrupalUid($this->currentUser()->id());

    dsm($form_state->getValues());

		$form['subscriptionjs_wrapper'] = [
			'#type' => 'item',
			'#markup' => '<div class="subscriptionjs-wrapper">xx</div>',
		];


    $form['actions']['submit'] = [
			'#type' => 'submit',
			'#value' => $this->t('Buy'),
			'#button_type' => 'primary',
		];

    return $form;
  }

	/**
	 * AJAX Callback for the plan variants details.
	 */
	public function updatePlanDetailsCallback(array &$form, FormStateInterface $form_state) {
		return $form['planvariant_details'];
	}

  public static function submitSelectionForm(array &$form, FormStateInterface $form_state) {
		$form_state->set('step', 'review')->setRebuild(TRUE);
	}

  public static function submitReviewForm(array &$form, FormStateInterface $form_state) {
		$form_state->set('step', 'buy')->setRebuild(TRUE);
	}

	/**
	 * {@inheritdoc}
	 */
	public function submitForm(array &$form, FormStateInterface $form_state) {
		// @todo Execute the required JS action or redirect.
		$planvariantMachineName = $form_state->getValue('planvariant');
		$planVariant = $this->subscriptionPlansManager->getBillwerkPlanVariantByMachineName($planvariantMachineName);

		$componentsMachineNames = array_filter($form_state->getValue('components'));
		/** @var \Drupal\billwerk_subscriptions_entities\Entity\BillwerkComponent[] $components */
		$components = [];
		if (!empty($componentsMachineNames)) {
			foreach ($componentsMachineNames as $componentMachineName) {
				$this->subscriptionPlansManager->getBillwerkComponentByMachineName($componentMachineName);
			}
		}
	}

	/**
	 * Checks access for a specific request.
	 *
	 * @param \Drupal\Core\Session\AccountInterface $account
	 *   Run access checks for this account.
	 *
	 * @return \Drupal\Core\Access\AccessResultInterface
	 *   The access result.
	 */
	public static function access(AccountInterface $account) {
		// @todo: Implement!
		return AccessResult::allowed();
	}

}
